local BaikCore = LibStub("AceAddon-3.0"):GetAddon("BaikCore")
local BaikUnitFrame = BaikCore.BaikUnitFrame
local Database = BaikCore.Database
local Hook = BaikCore.Hook
local Log = BaikCore.Log
local Reload = BaikCore.Reload
local Ui = BaikCore.Ui

local FrameStyle = BaikUnitFrame.FrameStyle
local OrderNumber = BaikUnitFrame.OrderNumber

-- Add Module
local ClassColorBar = BaikUnitFrame:NewModule("ClassColorBar", "AceEvent-3.0", "AceHook-3.0")
local PlayerFrameNameBackground = nil

-- Constants
local CLASS_COLOR_DB = "health_class_color"
local COLOR_DEFAULT = true

local NAME_BACKGROUND = {
    ["target"] = TargetFrameNameBackground,
    ["focus"] = FocusFrameNameBackground
}

-- Private Methods
local function IsEnabled()
    return Database:Get(CLASS_COLOR_DB)
end

local function GetColor(unit)
    if (UnitIsPlayer(unit) and IsEnabled()) then
        return Ui:GetClassColor(unit)
    else
        return Ui:GetUnitColor(unit)
    end
end
local function SetStatusBar(statusbar, unit)
    if not (unit and statusbar) then
        return
    end

    if (not statusbar.lockColor
        and not statusbar.disconnected) then
        local color = GetColor(unit)
        if color then
            statusbar:SetStatusBarColor(color.r, color.g, color.b)
        end
    end
end

local function SetClassTexture(texture, unit)
    if not (texture and unit) then
        return
    end

    local color = GetColor(unit)

    if color then
        texture:SetVertexColor(color.r, color.g, color.b)
    end
end

-- Hook Methods
local function SyncBlizz(statusbar, ...)
    local parent = statusbar:GetParent()
    SetClassTexture(parent.nameBackground, statusbar.unit)
end

local function SyncBlizzEvent(...)
    for unit, texture in pairs(NAME_BACKGROUND) do
        SetClassTexture(texture, unit)
    end
end

local function SyncBaik(statusbar, ...)
    SetStatusBar(statusbar, statusbar.unit)
end

local function CreatePlayerFrameNameBackground()
    local texture = PlayerFrame:CreateTexture("PlayerFrameNameBackground", BORDER)
    local color = Ui:GetClassColor("player")

    texture:SetTexture("Interface\\TargetingFrame\\UI-TargetingFrame-LevelBackground")
    texture:SetSize(119, 19)
    texture:SetPoint("TOPLEFT", 106, -22)
    if color then
        texture:SetVertexColor(color.r, color.g, color.b, 1)
    end

    return texture
end

local function HeaderOpts()
    return {
        name = "ClassColorBar",
        type = "header",
        order = OrderNumber:GetNext(),
    }
end

local function HealthOpts()
    return {
        name = "Class Color",
        type = "toggle",
        order = OrderNumber:GetNext(),
        get = IsEnabled,
        set = function(info, value, ...)
            Database:Set(CLASS_COLOR_DB, value)
            Reload:Enable()
        end
    }
end

-- Ace3 Callbacks
function ClassColorBar:OnInitialize()
    Log:v("ClassColorBar_OnInitialize")
    -- Initialize Database``
    Database:InitIfEmpty(CLASS_COLOR_DB, COLOR_DEFAULT)

    -- Add Options
    BaikUnitFrame:AddOption("health_bar_class", HealthOpts())
end

function ClassColorBar:OnEnable()
    Log:v("ClassColorBar_OnEnable")
    local sync = nil
    if not FrameStyle:IsBlizzard() then
        sync = SyncBaik
    elseif IsEnabled() then
        PlayerFrameNameBackground = CreatePlayerFrameNameBackground()
        sync = SyncBlizz
        self:RegisterEvent("GROUP_ROSTER_UPDATE", SyncBlizzEvent)
        self:RegisterEvent("PLAYER_TARGET_CHANGED", SyncBlizzEvent)
        self:RegisterEvent("PLAYER_FOCUS_CHANGED", SyncBlizzEvent)
        self:RegisterEvent("UNIT_FACTION", SyncBlizzEvent)
    else
        -- Use default blizzard
        return
    end
    Hook:EnsureSecureHook(ClassColorBar, "UnitFrameHealthBar_Update", sync)
    Hook:EnsureSecureHook(ClassColorBar, "HealthBar_OnValueChanged", sync)
end

function ClassColorBar:OnDisable()
    Log:v("ClassColorBar_OnDisable")
    self:UnhookAll()
end
