local BaikCore = LibStub("AceAddon-3.0"):GetAddon("BaikCore")
local BaikUnitFrame = BaikCore.BaikUnitFrame
local Assert = BaikCore.Assert
local CombatSafe = BaikCore.CombatSafe
local Database = BaikCore.Database
local Log = BaikCore.Log
local Reload = BaikCore.Reload

local OrderNumber = BaikUnitFrame.OrderNumber

-- Add Module
local FrameStyle = BaikUnitFrame:NewModule("FrameStyle")
BaikUnitFrame.FrameStyle = FrameStyle

-- Constants
local FRAME_STYLE_DB = "unit_frame_style"

local LARGE_HEALTH_TEXTURE = {
    ["Interface\\CharacterFrame\\UI-Player-Status"] = "Interface\\Addons\\BaikUnitFrame\\Textures\\UI-Player-Status",
    ["Interface\\TargetingFrame\\UI-TargetingFrame"] = "Interface\\TargetingFrame\\TargetFrame",
    ["Interface\\TargetingFrame\\UI-TargetingFrame-Elite"] = "Interface\\TargetingFrame\\TargetFrameElite",
    ["Interface\\TargetingFrame\\UI-TargetingFrame-Rare"] = "Interface\\TargetingFrame\\TargetFrameRare",
    ["Interface\\TargetingFrame\\UI-TargetingFrame-RareElite"] = "Interface\\TargetingFrame\\TargetFrameRareElite",
    ["Interface\\TargetingFrame\\UI-UnitFrame-Boss"] = "Interface\\TargetingFrame\\UnitFrameBoss"
}

local LARGE_HEALTH_KEY = 1
local COMPACT_KEY = 2
local BLIZZARD_KEY = 3


local FRAME_OPTS = {
    "Large Health",
    "Compact",
    "Blizzard"
}

-- Private Methods
local function HeaderOpt()
    return {
        name = "Frame Style",
        type = "header",
        order = OrderNumber:GetNext()
    }
end

local function FrameOpts()
    return {
        name = "Frame Style",
        type = "select",
        values = FRAME_OPTS,
        order = OrderNumber:GetNext(),
        get = function(...)
            return Database:Get(FRAME_STYLE_DB)
        end,
        set = function(self, key, value, ...)
            local curState = Database:Get(FRAME_STYLE_DB)

            if key ~= curState then
                Database:Set(FRAME_STYLE_DB, key)
                Reload:Enable()
            end
        end
    }
end

-- FrameStyle API
function FrameStyle:GetTexture(texture)
    Assert:String(texture)
    local frameStyle = Database:Get(FRAME_STYLE_DB)
    if frameStyle == LARGE_HEALTH_KEY then
        local newTexture = LARGE_HEALTH_TEXTURE[texture]

        if not newTexture then
            newTexture = texture
        end

        texture = newTexture
    end

    return texture
end

function FrameStyle:IsLarge()
    return Database:Get(FRAME_STYLE_DB) == LARGE_HEALTH_KEY
end

function FrameStyle:IsCompact()
    return Database:Get(FRAME_STYLE_DB) == COMPACT_KEY
end

function FrameStyle:IsBlizzard()
    return Database:Get(FRAME_STYLE_DB) == BLIZZARD_KEY
end

-- Ace3 Callbacks
function FrameStyle:OnInitialize()
    Log:v("FrameStyle_OnInitialize")
    Database:InitIfEmpty(FRAME_STYLE_DB, COMPACT_KEY)
    BaikUnitFrame:AddOption("frame_style", FrameOpts())
end

function FrameStyle:OnEnable()
    Log:v("FrameStyle_OnEnable")
end

function FrameStyle:OnDisable()
    Log:v("FrameStyle_OnDisable")
end
