local BaikCore = LibStub("AceAddon-3.0"):GetAddon("BaikCore")
local BaikUnitFrame = BaikCore.BaikUnitFrame
local Database = BaikCore.Database
local LayoutAttrib = BaikCore.LayoutAttrib
local Log = BaikCore.Log

local FrameStyle = BaikUnitFrame.FrameStyle

-- Add Module
local PlayerFrameMod = BaikUnitFrame:NewModule("PlayerFrameMod", "AceHook-3.0")

-- UI Layout that gets moved by Blizzard
local PLAYER_TABLE = {
    [PlayerName] = LayoutAttrib:Create(100, 12, "CENTER", 50, 36),
    [PlayerFrameHealthBar] = LayoutAttrib:Create(119, 19, "TOPLEFT", 106, -22),
    [PlayerFrameManaBar] = LayoutAttrib:Create(119, 12, "TOPLEFT", 106, -41),
    [PlayerFrameHealthBarText] = LayoutAttrib:Create(nil, nil, "CENTER", 50, 18),
    [PlayerFrameHealthBarTextLeft] = LayoutAttrib:Create(nil, nil, "LEFT", 110, 18),
    [PlayerFrameHealthBarTextRight] = LayoutAttrib:Create(nil, nil, "RIGHT", -8, 18),
    [PlayerFrameManaBarText] = LayoutAttrib:Create(nil, nil, "CENTER", 50, 3),
    [PlayerFrameManaBarTextLeft] = LayoutAttrib:Create(nil, nil, "LEFT", 110, 3),
    [PlayerFrameManaBarTextRight] = LayoutAttrib:Create(nil, nil, "RIGHT", -8, 3),
}

local LARGE_HEALTH_TABLE = {
    [PlayerFrameHealthBar] = LayoutAttrib:Create(119, 31, "TOPLEFT", 106, -22),
    [PlayerFrameHealthBarText] = LayoutAttrib:Create(nil, nil, "CENTER", 50, 7),
    [PlayerFrameHealthBarTextLeft] = LayoutAttrib:Create(nil, nil, "LEFT", 110, 7),
    [PlayerFrameHealthBarTextRight] = LayoutAttrib:Create(nil, nil, "RIGHT", -8, 7),
    [PlayerStatusTexture] = LayoutAttrib:Create(nil, nil, "TOPLEFT", 36, -9),
}

local LARGE_TEXTURE = {
    PlayerFrameTexture,
    PlayerStatusTexture
}

-- Private Methods
local function LoadPlayer()
    if FrameStyle:IsCompact() then
        for frame, layout in pairs(PLAYER_TABLE) do
            layout:SyncRegion(frame, true)
        end
    end

    if FrameStyle:IsLarge() then
        for frame, layout in pairs(LARGE_HEALTH_TABLE) do
            layout:SyncRegion(frame, true)
        end

        for _, texture in pairs(LARGE_TEXTURE) do
            local curTexture = texture:GetTexture()
            local newTexture = FrameStyle:GetTexture(curTexture)
            texture:SetTexture(newTexture)
        end
    end
end

-- PostHook Methods
local function PostHook_PlayerFrame_ToPlayerArt(...)
    LoadPlayer()
end

local function PostHook_PlayerFrame_ToVehicleArt(...)
    PlayerFrameTexture:Show()
    PlayerFrame_HideVehicleTexture()
    LoadPlayer()
end

-- Ace3 Callbacks
function PlayerFrameMod:OnInitialize()
    Log:v("PlayerFrameMod_OnInitialize")
end

function PlayerFrameMod:OnEnable()
    Log:v("PlayerFrameMod_OnEnable")
    if FrameStyle:IsBlizzard() then
        return
    end

    LoadPlayer()
    self:SecureHook("PlayerFrame_ToPlayerArt", PostHook_PlayerFrame_ToPlayerArt)
    self:SecureHook("PlayerFrame_ToVehicleArt", PostHook_PlayerFrame_ToVehicleArt)
end

function PlayerFrameMod:OnDisable()
    Log:v("PlayerFrameMod_OnDisable")
    self:UnhookAll()
end
