local BaikCore = LibStub("AceAddon-3.0"):GetAddon("BaikCore")
local BaikUnitFrame = BaikCore.BaikUnitFrame
local Database = BaikCore.Database
local Hook = BaikCore.Hook
local Log = BaikCore.Log
local RootStatusBar = BaikCore.StatusBar

local OrderNumber = BaikUnitFrame.OrderNumber

-- Add Module
local StatusBar = BaikUnitFrame:NewModule("StatusBar", "AceHook-3.0")

-- Constants
local UNIT_TEXTURE_DB = "unit_status_bar"


local function HeaderOpts()
    return {
        name = "StatusBar",
        type = "header",
        order = OrderNumber:GetNext(),
    }
end
local function BarOpts()
    return RootStatusBar:GetOpts(UNIT_TEXTURE_DB)
end

local STATUSBARS = {
    -- Player
    PlayerFrameHealthBar,
    PlayerFrameManaBar,
    PlayerFrameAlternateManaBar,
    CastingBarFrame,
    -- Target
    TargetFrameHealthBar,
    TargetFrameManaBar,
    TargetFrameSpellBar,
    -- Focus
    FocusFrameHealthBar,
    FocusFrameManaBar,
    FocusFrameSpellBar,
    --Party
    PartyMemberFrame1HealthBar,
    PartyMemberFrame1ManaBar,
    PartyMemberFrame2HealthBar,
    PartyMemberFrame2ManaBar,
    PartyMemberFrame3HealthBar,
    PartyMemberFrame3ManaBar,
    PartyMemberFrame4HealthBar,
    PartyMemberFrame4ManaBar,
    --Boss
    Boss1TargetFrameHealthBar,
    Boss1TargetFrameManaBar,
    Boss1TargetFrameSpellBar,
    Boss2TargetFrameHealthBar,
    Boss2TargetFrameManaBar,
    Boss2TargetFrameSpellBar,
    Boss3TargetFrameHealthBar,
    Boss3TargetFrameManaBar,
    Boss3TargetFrameSpellBar,
    Boss4TargetFrameHealthBar,
    Boss4TargetFrameManaBar,
    Boss4TargetFrameSpellBar,
    Boss5TargetFrameHealthBar,
    Boss5TargetFrameManaBar,
    Boss5TargetFrameSpellBar,
}

-- Private Methods
local function PostHook_UnitFrameManaBar_UpdateType(manaBar, ...)
    RootStatusBar:SyncBar(manaBar, UNIT_TEXTURE_DB)
end

-- Ace3 Callbacks
function StatusBar:OnInitialize()
    Log:v("StatusBar_OnInitialize")
    Database:InitIfEmpty(UNIT_TEXTURE_DB, RootStatusBar.DEFAULT)
    BaikUnitFrame:AddOption("statusbar_header", HeaderOpts())
    BaikUnitFrame:AddOption("statusbar_style", BarOpts())
    RootStatusBar:AddBars(STATUSBARS, UNIT_TEXTURE_DB)
end

function StatusBar:OnEnable()
    Log:v("StatusBar_OnEnable")
    Hook:EnsureSecureHook(self, "UnitFrameManaBar_UpdateType",
                          PostHook_UnitFrameManaBar_UpdateType)
    RootStatusBar:Sync(UNIT_TEXTURE_DB)
end

function StatusBar:OnDisable()
    Log:v("StatusBar_OnDisable")
    Hook:EnsureUnhook(self, "UnitFrameManaBar_UpdateType")
end
